package com.ef_prime.rflow;

import java.io.File;
import java.util.Locale;

import com.ef_prime.rflow.util.ExtensionUtility;

public class Constants {
	
	public static final String PROGRAM_DIR = new File("").getAbsolutePath(); //$NON-NLS-1$
	public static final String PROGRAM_DIR_SLASH = replaceFileSeparator(PROGRAM_DIR)+'/';
	
	public static final String RFLOW_FILE_EXTENSION = ".rflow"; //$NON-NLS-1$

	
	/**
	 * Rflow用の環境の名前
	 */
	public static final String RFLOW_ENV = ".RflowEnv"; //$NON-NLS-1$
	
	/**
	 * Ｒ用の改行文字
	 */
	public static final char NEWLINE = '\n';
	public static final String NEWLINE_STR = "\n"; //$NON-NLS-1$
	
	/** //TODO コメント文字は変更できるのでＲから取得するようにする */
	public static final char COMMENT_CHAR = '#';
	
	/**
	 * Ｒ用のファイルパスの区切り文字
	 */
	public static final char FILE_SEPARATOR_CHAR = '/';
	public static final String FILE_SEPARATOR = "/"; //$NON-NLS-1$
	
	public static final String QUOTE = "\""; //$NON-NLS-1$
	
	
	
	/**
	 * 両端を"で囲み、"をエスケープします
	 * @param s
	 * @return
	 */
	public static String quote(String s){
		String quoted = s.replaceAll("\"", "\\\\\""); //$NON-NLS-1$ //$NON-NLS-2$
		return QUOTE + quoted + QUOTE;
	}
	
	public static String unquote(String s){
		if(s.startsWith(QUOTE) && s.endsWith(QUOTE) && s.length() > 1){
			if(s.length() == 2)
				return ""; //$NON-NLS-1$
			s = s.substring(1, s.length()-1);
			return s.replaceAll("\\\\\"", "\""); //$NON-NLS-1$ //$NON-NLS-2$
		}
		return s;
	}
	
	/**
	 * Ｒ用のlogicalの文字列をbooleanに変換します
	 * @param s
	 * @return
	 */
	public static boolean toBoolean(String s){
		if(s.equals("TRUE") || s.equals("T")) //$NON-NLS-1$ //$NON-NLS-2$
			return true;
		if(s.equals("FALSE") || s.equals("F")) //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		throw new IllegalArgumentException("can not parse to boolean"); //$NON-NLS-1$
	}
	
	/**
	 * booleanからＲ用のlogicalの文字列に変換します
	 * @param b
	 * @return
	 */
	public static String toRBoolean(boolean b){
		return (b)?"TRUE":"FALSE"; //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	public static String combine(String[] objects){
		String ret = "c("; //$NON-NLS-1$
		for(String o : objects){
			ret += o+", "; //$NON-NLS-1$
		}
		ret = ret.substring(0, ret.length()-2);
		ret += ")"; //$NON-NLS-1$
		return ret;
	}
	
	/**
	 * ロケールの言語名がついたファイルを返します。
	 * もし、ロケールの言語名のついたファイルが存在しなかった場合は、
	 * 元のファイルを返します。
	 * help.chm -> help_ja.chm
	 * @param file
	 * @return
	 */
	public static File getLocaleFile(File file){
		String name = ExtensionUtility.removeExtension(file.getName());
		String ext = ExtensionUtility.getExtension(file.getName());
		
		//String lang = Locale.getDefault().getLanguage();
		String lang = Locale.getDefault().getLanguage();
		File localFile = new File(file.getParent(), name+'_'+lang+'.'+ext);
		
		return (localFile.exists())?localFile:file;
	}
	
	/**
	 * プログラムからの相対パスを絶対パスに変換する
	 * @param relativePath
	 * @return
	 */
	public static String getPath(String relativePath){
		return PROGRAM_DIR_SLASH+relativePath;
	}
	
	public static File getUserFile(String filename){
		return new File(Platform.getUserDir(), filename);
	}
	
	/**
	 * システムの区切り文字をR用の区切り文字に変換する
	 * @param s
	 * @return
	 */
	public static String replaceFileSeparator(String s){
		return s.replace(File.separatorChar, Constants.FILE_SEPARATOR_CHAR);
	}
	
	/**
	 * R用の区切り文字をシステムの区切り文字に変換する
	 * @param s
	 * @return
	 */
	public static String replaceRSep2SystemSep(String s){
		return s.replace(Constants.FILE_SEPARATOR_CHAR, File.separatorChar);
	}
}
