package com.ef_prime.rflow.flow;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import com.ef_prime.rflow.flow.model.FlowEdge;
import com.ef_prime.rflow.flow.model.FlowGraph;
import com.ef_prime.rflow.flow.model.FlowNode;
import com.ef_prime.rflow.flow.model.FlowGraph.GraphChangeType;
import com.ef_prime.rflow.node.NodeModel;
import com.ef_prime.rflow.node.NodeModelChangeListener;

import edu.uci.ics.jung.visualization.MultiPickedState;

public class FlowState extends MultiPickedState implements FlowGraph.GraphWillChangeListener, NodeModelChangeListener{
	
	private FlowNode flagNode;
	private FlowNode highlightNode;
	//private static final int MAX_PIN = 2;
	//private LinkedList<FlowNode> pins = new LinkedList<FlowNode>(); 
	private List<FlowNode> selectionFlow = new ArrayList<FlowNode>();
	private List<FlowEdge> selectionEdgeFlow = new ArrayList<FlowEdge>();
	
	private FlowEdge insertTargetEdge;
	
	private List<FlagNodeChangeListener> flagNodeChangeListeners = new ArrayList<FlagNodeChangeListener>();
	
	public FlowState(){
		super();
		addItemListener(new StreamSelectionHandler());
	}
	
	public void clearAllState(){
		clearPickedVertices();
		clearPickedEdges();
		clearInsertTargetEdge();
		flagNode = null;
		highlightNode = null;
		//clearPinNodes();
	}
	
	public void pickVertices(Iterable<FlowNode> nodes, boolean picked){
		for(FlowNode node : nodes)
			pick(node, picked);
	}
	
	public void pickEdges(Iterable<FlowEdge> edges, boolean picked){
		for(FlowEdge edge : edges)
			pick(edge, picked);
	}
	
	/**
	 * 選択されているノードを一つ返す。
	 * 複数選択されている、もしくは一つも選択されていない時はnullが返ります。
	 * @return
	 */
	public FlowNode getSelectedNode(){
		if(getPickedVertices().size() != 1)
			return null;
		return (FlowNode)getPickedVertices().iterator().next();
	}
	
	@SuppressWarnings("unchecked") //$NON-NLS-1$
	@Override
	public Set<FlowNode> getPickedVertices() {
		return super.getPickedVertices();
	}
	
	@SuppressWarnings("unchecked") //$NON-NLS-1$
	@Override
	public Set<FlowEdge> getPickedEdges() {
		return super.getPickedEdges();
	}

	public List<FlowNode> getSelectionNodeFlow() {
		return (List<FlowNode>)Collections.unmodifiableList(selectionFlow);
	}
	
	public List<FlowEdge> getSelectionEdgeFlow() {
		return (List<FlowEdge>)Collections.unmodifiableList(selectionEdgeFlow);
	}


	private class StreamSelectionHandler implements ItemListener{
		public void itemStateChanged(ItemEvent e) {
			selectionFlow.clear();
			selectionEdgeFlow.clear();
			if(getPickedVertices().size() == 1){
				List<FlowNode> stream = FlowUtil.getStream((FlowNode)getPickedVertices().iterator().next());
				selectStream(stream);
			}
		}
	}
	
	private void selectStream(List<FlowNode> stream){
		FlowNode parent = null;
		for(FlowNode node : stream){
			selectionFlow.add(node);
			if(parent != null)
				selectionEdgeFlow.add((FlowEdge)parent.findEdge(node));
			
			parent = node;
		}
	}
	
	public void addFlagNodeChageListener(FlagNodeChangeListener changeListener){
		flagNodeChangeListeners.add(changeListener);
	}

	public FlowNode getFlagNode() {
		return flagNode;
	}

	public void setFlagNode(FlowNode flagNode) {
		if(this.flagNode != null && flagNode != null && this.flagNode.equals(flagNode))
			return;
		this.flagNode = flagNode;
		for(FlagNodeChangeListener l : flagNodeChangeListeners){
			l.flagNodeChanged(flagNode);
		}
	}
	
	public boolean isFlagNode(FlowNode node){
		if(flagNode == null)return false;
		return flagNode.equals(node);
	}
	
	/*NodeModelChangeListener*/
	public void nodeModelChanged(FlowNode node, NodeModel old, NodeModel model) {
		if(flagNode == null)return;
		List<FlowNode> stream = FlowUtil.getStream(flagNode);
		for(FlowNode nod : stream){
			if(nod == node)
				setFlagNode(null);
		}
	}

	public static interface FlagNodeChangeListener{
		void flagNodeChanged(FlowNode newFlagNode);
	}
	
	public FlowNode getHighlightNode() {
		return highlightNode;
	}
	
	public void setHighlightNode(FlowNode highlightNode) {
		this.highlightNode = highlightNode;
	}
	
	public boolean isHighlightNode(FlowNode node) {
		if(highlightNode == null)return false;
		return highlightNode.equals(node);
	}
	
	/*public boolean isPinFlowSelected(){
		return pins.size() == MAX_PIN;
	}*/
	
	/*public boolean isPinNode(FlowNode pinNode){
		return pins.contains(pinNode);
	}*/

	/*public void addSelectionPin(FlowNode pinNode){
		if(isPinFlowSelected())
			clearPinNodes();
		pins.add(pinNode);
		while(pins.size() > MAX_PIN){
			pins.removeFirst();
		}
		//二つのピンの間を選択
		if(isPinFlowSelected()){
			List<FlowNode> firstStream = FlowUtil.getStream(pins.getFirst());
			List<FlowNode> lastStream = FlowUtil.getStream(pins.getLast());
			
			boolean rev = (firstStream.contains(pins.getLast()));
			if(!rev && !lastStream.contains(pins.getFirst())){
				pins.removeFirst();
				return;
			}
			
			FlowNode first = (!rev)?pins.getFirst():pins.getLast();
			List<FlowNode> stream = (!rev)?lastStream:firstStream;
			stream = stream.subList(stream.indexOf(first), stream.size());

			clearPickedVertices();
			selectionFlow.clear();
			selectionEdgeFlow.clear();
			selectStream(stream);
		}
	}*/
	
	/**
	 * 選択ノードからnodeまでのノードを選択する
	 * @param node
	 */
	public void selectInterval(FlowNode node){
		Set<FlowNode> nodes = getPickedVertices();
		if(nodes.size() == 1){
			FlowNode start = nodes.iterator().next();
			
			List<FlowNode> firstStream = FlowUtil.getStream(start);
			List<FlowNode> lastStream = FlowUtil.getStream(node);
			
			boolean rev = (firstStream.contains(node));
			//二つノードが区間を作らない場合
			if(!rev && !lastStream.contains(start)){
				clearPickedVertices();
				pick(node, true);
				return;
			}
			
			FlowNode first = (!rev)?start:node;
			List<FlowNode> stream = (!rev)?lastStream:firstStream;
			stream = stream.subList(stream.indexOf(first), stream.size());

			clearPickedVertices();
			for(FlowNode select : stream)
				pick(select, true);
		}else{
			clearPickedVertices();
			pick(node, true);
		}
	}
	
	/*public List<FlowNode> getPinNodes(){
		return Collections.unmodifiableList(pins);
	}*/
	
	/*public void clearPinNodes(){
		if(isPinFlowSelected()){
			selectionFlow.clear();
			selectionEdgeFlow.clear();
		}
		pins.clear();
	}*/
	
	public void clearInsertTargetEdge(){
		insertTargetEdge = null;
	}
	
	public void setInsertTargetEdge(FlowEdge insertTargetEdge) {
		this.insertTargetEdge = insertTargetEdge;
	}
	
	public FlowEdge getInsertTargetEdge() {
		return insertTargetEdge;
	}
	
	/*----------------------StreamGraph.GraphWillChangeListener----------------------*/
	public void graphWillChange(Object graphElement, GraphChangeType type, FlowGraph graph) {
		if(flagNode == null)return;
		List<FlowNode> stream = FlowUtil.getStream(flagNode);
		switch(type){
		case ADD:
			if(graphElement instanceof FlowEdge){
				//先頭のチェックだけでよい
				if(stream.get(0) == ((FlowEdge)graphElement).getDest()){
					setFlagNode(null);
					break;
				}
			}
			break;
		case REMOVE:
			if(graphElement instanceof FlowEdge){
				for(FlowNode node : stream){
					if(!isFlagNode(node) && node.isSource((FlowEdge)graphElement)){
						setFlagNode(null);
						break;
					}
				}
			}
			else if(graphElement instanceof FlowNode){
				for(FlowNode node : stream){
					if(node.equals((FlowNode)graphElement)){
						setFlagNode(null);
						break;
					}
				}
			}
			break;
		}
		
	}
	
	@Override
	protected void fireItemStateChanged(ItemEvent e) {
        Object[] listeners = listenerList.getListeners(ItemListener.class);//逆順に渡される
        for (int i = listeners.length-1; i >= 0; i--) {
        	((ItemListener)listeners[i]).itemStateChanged(e);
        }
	}
}
