package com.ef_prime.rflow.flow;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.swing.undo.CompoundEdit;

import org.apache.commons.collections.Buffer;
import org.apache.commons.collections.buffer.UnboundedFifoBuffer;

import com.ef_prime.rflow.Constants;
import com.ef_prime.rflow.flow.model.FlowEdge;
import com.ef_prime.rflow.flow.model.FlowGraph;
import com.ef_prime.rflow.flow.model.FlowNode;

import edu.uci.ics.jung.graph.ArchetypeVertex;
import edu.uci.ics.jung.graph.Edge;
import edu.uci.ics.jung.graph.Vertex;

public class FlowUtil {

	/**
	 * ルートからnodeまで順番に入っているList
	 * @param node
	 * @return
	 */
	public static List<FlowNode> getStream(FlowNode node){
		List<FlowNode> stream = new ArrayList<FlowNode>();
		getStream(node, stream);
		return stream;
	}
	
	@SuppressWarnings("unchecked") //$NON-NLS-1$
	private static void getStream(FlowNode node, List<FlowNode> stream){
		stream.add(0, node);
		
		Set<Edge> inEdge = node.getInEdges();
		if(inEdge.size() == 1){
			Edge e = inEdge.iterator().next();
			getStream((FlowNode)e.getOpposite(node), stream);
		}
	}
	
	public static String getCommandFromStream(List<FlowNode> stream, boolean comment){
		StringBuffer sb = new StringBuffer();
		for(FlowNode node : stream){
			if(comment){
				String commentStr = node.getModel().getProperty().getComment();
				if(commentStr != null && commentStr.length() > 0){
					String[] lines = commentStr.split(Constants.NEWLINE_STR, -1);
					for(int i = 0; i< lines.length; i++){
						if(i == lines.length-1 && lines[i].equals("")) //$NON-NLS-1$
							break;
						sb.append(Constants.COMMENT_CHAR+lines[i]+Constants.NEWLINE);
					}
				}
			}
			sb.append(node.getModel().getCommandNewline());
		}
		return sb.toString();
	}
	/*
	public static String getFlowCommand(FlowNode node, boolean comment){
		List<FlowNode> stream = getStream(node);
		StringBuffer sb = new StringBuffer();
		getCommand(stream.remove(0), sb, stream, true, comment);
		return sb.toString();
	}
	
	private static void getCommand(
			FlowNode node, 
			StringBuffer sb, 
			List<FlowNode> mainStream,
			boolean isMain,
			boolean comment)
	{
		if(comment){
			String commentStr = node.getModel().getProperty().getComment();
			if(commentStr != null && commentStr.length() > 0){
				String[] lines = commentStr.split(Constants.NEWLINE_STR);
				for(String l : lines)
					sb.append(Constants.COMMENT_CHAR+l+Constants.NEWLINE);
			}
		}
		
		sb.append(node.getModel().getCommandNewline());
		
		FlowNode nextMain = null;
		if(isMain && mainStream.size()>0)
			nextMain = mainStream.remove(0);
		
		for(Object o : node.getSuccessors()){
			FlowNode fn = (FlowNode)o;
			boolean main = fn.equals(nextMain);
			
			if(!main)sb.append("if(FALSE){");
			getCommand(fn, sb, mainStream, main, comment);
			if(!main)sb.append("}");
		}
	}*/
	
	
	public static boolean isEdgeAddable(FlowEdge edge){
		Vertex dest = (Vertex)edge.getDest();
		
		//親が複数いる場合
		if(dest.getPredecessors().size() > 0)return false;

		return checkLoop((Vertex)edge.getSource(), dest);
	}

	/**
	 * 親が複数いる場合、元の親へのエッジを消去する。
	 * 後は、isEdgeAddableと同じ
	 * @param edge
	 * @return 追加可能になったか
	 */
	public static boolean makeEdgeAddable(FlowEdge edge, FlowViewer viewer, CompoundEdit edit){
		Vertex dest = (Vertex)edge.getDest();
		
		//親が複数いる場合
		if(dest.getPredecessors().size() > 0){
			FlowGraph graph = (FlowGraph)dest.getGraph();
			Set<FlowEdge> edges = new HashSet<FlowEdge>();
			for(Object o : dest.getInEdges()){
				FlowEdge e = (FlowEdge)o;
				graph.removeEdge(e);
				edges.add(e);
			}
			
			edit.addEdit(viewer.undoableEditFactory.createDeleteEdit(null, edges.toArray(new FlowEdge[0])));
		}

		return checkLoop((Vertex)edge.getSource(), dest);
	}

	private static boolean checkLoop(Vertex src, Vertex dest) {
		//循環している場合
		while(src.getPredecessors().size() > 0){
			src = (Vertex)src.getPredecessors().iterator().next();
			if(src.equals(dest))
				return false;
		}
		return true;
	}
	
	
	
	
	/**
	 * インアウトノードの組をを取得する
	 * @param nodeSet
	 * @return
	 */
	public static NodeSelectionInfo[] getNodeSelectionInfos(FlowNode[] nodes){
		Set<FlowNode> nodeSet = new HashSet<FlowNode>(Arrays.asList(nodes));
		
		//クラスタに分解
		List<Set<FlowNode>> clusters = extractCluster(nodeSet);
		
		//クラスタのルート取得
		List<FlowNode> sources = new ArrayList<FlowNode>();
		for(Set<FlowNode> cluster : clusters){
			FlowNode aNode = cluster.iterator().next();
			sources.add(getSource(aNode, cluster));
		}
		
		Set<NodeSelectionInfo> infoSet = new HashSet<NodeSelectionInfo>();
		for(int i = 0; i < clusters.size(); i++){
			FlowNode src = sources.get(i);
			if(src.getPredecessors().isEmpty()) continue;
			
			FlowNode in = (FlowNode)src.getPredecessors().iterator().next();
			if(in == null)continue;
			
			Set<FlowNode> outs = new HashSet<FlowNode>();
			getOutOfNodes(src, outs, clusters.get(i));
			if(outs.isEmpty())continue;
			
			for(FlowNode out : outs)
				infoSet.add(new NodeSelectionInfo(in, out, src));
		}
		
		return infoSet.toArray(new NodeSelectionInfo[0]);
	}
	
	private static FlowNode getSource(FlowNode target, Set<FlowNode> nodes){
		if(target.getPredecessors().isEmpty())
			return target;
		FlowNode source = (FlowNode)target.getPredecessors().iterator().next();
		if(nodes.contains(source))
			return getSource(source, nodes);
		else
			return target;
	}
	
	private static void getOutOfNodes(FlowNode target, Set<FlowNode> out, Set<FlowNode> nodes){
		if(target.getSuccessors().isEmpty())
			return;
		for(Object o : target.getSuccessors()){
			FlowNode dest = (FlowNode)o;
			if(nodes.contains(dest))
				getOutOfNodes(dest, out, nodes);
			else
				out.add(dest);
		}
	}
	
	
	@SuppressWarnings("unchecked") //$NON-NLS-1$
	public static List<Set<FlowNode>> extractCluster(Set<FlowNode> nodes) {

		List<Set<FlowNode>> clusters = new ArrayList<Set<FlowNode>>();
		
        HashSet<FlowNode> unvisitedVertices = new HashSet<FlowNode>(nodes);

        while (!unvisitedVertices.isEmpty()) {
            Set<FlowNode> weakComponentSet = new HashSet<FlowNode>();
            FlowNode root = (FlowNode) unvisitedVertices.iterator().next();
            unvisitedVertices.remove(root);
            weakComponentSet.add(root);

            Buffer queue = new UnboundedFifoBuffer();
            queue.add(root);

            while (!queue.isEmpty()) {
                ArchetypeVertex currentVertex = (ArchetypeVertex) queue.remove();
                Set neighbors = currentVertex.getNeighbors();

                for (Iterator nIt = neighbors.iterator(); nIt.hasNext();) {
                	FlowNode neighbor = (FlowNode) nIt.next();
                    if (unvisitedVertices.contains(neighbor)) {
                        queue.add(neighbor);
                        unvisitedVertices.remove(neighbor);
                        weakComponentSet.add(neighbor);
                    }
                }
            }
            clusters.add(weakComponentSet);
        }

        return clusters;
    }

	public static void takeOut(FlowNode[] nodes, boolean branch,
			UndoableEditFactory factory, CompoundEdit edit)
	{
		NodeSelectionInfo[] infos = getNodeSelectionInfos(nodes);
		for(NodeSelectionInfo info : infos){
			takeOut(info, branch, factory, edit);
		}
	}
	
	public static void takeOut(NodeSelectionInfo info, boolean branch, 
			UndoableEditFactory factory, CompoundEdit edit)
	{
		FlowGraph graph = (FlowGraph)info.getSource().getGraph();
		
		if(!branch){
			Edge e1 = info.getIn().findEdge(info.getSource());
			if(e1 != null && graph.getEdges().contains(e1)){
				graph.removeEdge(e1);
				edit.addEdit(factory.createDeleteEdit(null, new FlowEdge[]{(FlowEdge)e1}));
			}
		}
		
		Set inSet = info.getOut().getInEdges();
		if(!inSet.isEmpty()){
			Edge e2 = (Edge)inSet.iterator().next();
			if(graph.getEdges().contains(e2)){
				graph.removeEdge(e2);
				edit.addEdit(factory.createDeleteEdit(null, new FlowEdge[]{(FlowEdge)e2}));
			}
		}
		
		FlowEdge newEdge1 = new FlowEdge(info.getIn(), info.getOut());
		if(isEdgeAddable(newEdge1)){
			graph.addEdge(newEdge1);
			edit.addEdit(factory.createAddEdgeEdit(new FlowEdge[]{newEdge1}));
		}
	}
	
	/**
	 * 子ノードを取得。自分自身も含む。
	 * @param parents
	 */
	public static Set<FlowNode> getChildren(FlowNode[] parents){

		Set<FlowNode> children = new HashSet<FlowNode>();
		
        HashSet<FlowNode> currentNodes = new HashSet<FlowNode>(Arrays.asList(parents));        

        while (!currentNodes.isEmpty()) {
            FlowNode node = (FlowNode) currentNodes.iterator().next();
            currentNodes.remove(node);
            if(!children.contains(node)){
            	children.add(node);
            	for(Object o : node.getSuccessors()){
            		FlowNode child = (FlowNode)o;
            		currentNodes.add(child);
            	}
            }            
        }

        return children;
	}
}
