package com.ef_prime.rflow.flow.model;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import edu.uci.ics.jung.graph.Edge;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.impl.SparseGraph;

public class FlowGraph extends SparseGraph{
	
	List<GraphWillChangeListener> graphWillChangeListener = new ArrayList<GraphWillChangeListener>();
	
	
	
	@Override
	public Vertex addVertex(Vertex v) {
		fireGraphWillChange(v, GraphChangeType.ADD, this);
		return super.addVertex(v);
	}
	
	@Override
	public Edge addEdge(Edge e) {
		fireGraphWillChange(e, GraphChangeType.ADD, this);
		return super.addEdge(e);
	}

	@Override
	public void removeEdge(Edge e) {
		fireGraphWillChange(e, GraphChangeType.REMOVE, this);
		super.removeEdge(e);
	}
	
	@Override
	public void removeEdges(Set edges) {
		super.removeEdges(edges);
	}
	
	@Override
	public void removeAllEdges() {
		super.removeAllEdges();
	}
	
	@Override
	public void removeVertex(Vertex v) {
		fireGraphWillChange(v, GraphChangeType.REMOVE, this);
		super.removeVertex(v);
	}
	
	@Override
	public void removeVertices(Set vertices) {
		super.removeVertices(vertices);
	}
	
	@Override
	public void removeAllVertices() {
		for(Object o : getVertices()){
			fireGraphWillChange((FlowNode)o, GraphChangeType.REMOVE, this);
		}
		super.removeAllVertices();
	}
	
	public void addGraphWillChangeListener(GraphWillChangeListener l){
		graphWillChangeListener.add(l);
	}
	
	private void fireGraphWillChange(Object graphElement, GraphChangeType type, FlowGraph graph){
		for(GraphWillChangeListener l : graphWillChangeListener){
			l.graphWillChange(graphElement, type, graph);
		}
	}
	
	/**
	 * グラフが変更される直前に投げられるイベントに対するリスナー
	 *
	 */
	public static interface GraphWillChangeListener{
		void graphWillChange(Object graphElement, GraphChangeType type, FlowGraph graph);
	}
	
	public static enum GraphChangeType{
		ADD,REMOVE;
	}
}
