package com.ef_prime.rflow.node;

import java.awt.Component;
import java.awt.Dimension;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;

import com.ef_prime.rflow.group.AppearanceGroup;
import com.ef_prime.rflow.group.DefaultAppearanceGroup;
import com.ef_prime.rflow.resource.Messages;

class NodePropertyPanel extends JPanel{
	
	private NodeModel model;
	private List<ModelUpdater> modelUpdaters = new ArrayList<ModelUpdater>();
	
	NodePropertyPanel(final NodeModel model) {
		super();
		this.model = model;
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		int space = 20;
		setBorder(new EmptyBorder(space,space,space,space));
		
		addPropertyPanel(new AppearancePanel());
		addPropertyPanel(new CommentPanel());
	}
	
	private void addPropertyPanel(AbstractPropertyPanel propertyPanel){
		modelUpdaters.add(propertyPanel);
		propertyPanel.setModel(model);
		add(propertyPanel);
	}
	
	void setPropertiesToModel(NodeModel model){
		for(ModelUpdater updater : modelUpdaters)
			updater.updateModel(model);
	}
	
	private interface ModelUpdater{
		void setModel(NodeModel model);
		void updateModel(NodeModel model);
	}
	
	private abstract class AbstractPropertyPanel extends JPanel implements ModelUpdater{
	}
	
	private class AppearancePanel extends AbstractPropertyPanel{
		
		private JComboBox comboBox;
		
		public AppearancePanel() {
			super();
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			setBorder(new TitledBorder(Messages.getString("NodePropertyPanel.node_shape"))); //$NON-NLS-1$
			
			Vector<AppearanceGroup> groupVector = new Vector<AppearanceGroup>();
			groupVector.add(null);
			groupVector.addAll(Arrays.asList(DefaultAppearanceGroup.getAppearanceGroups()));
			comboBox = new JComboBox(groupVector);
			comboBox.setRenderer(new DefaultListCellRenderer(){
				@Override
				public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
					super.getListCellRendererComponent(list, value, index, isSelected,
							cellHasFocus);
					
					
					AppearanceGroup group = (AppearanceGroup)value;
					if(group == null){
						setText(Messages.getString("NodePropertyPanel.default")); //$NON-NLS-1$
					}else{
						setIcon(group.getSmallIcon());
					}
					
					return this;
				}
			});
			comboBox.setMaximumSize(new Dimension(Integer.MAX_VALUE, 25));
			add(comboBox);
		}
		
		public void setModel(NodeModel model) {
			comboBox.setSelectedItem(model.getProperty().getAppearanceGroup());
		}
		
		public void updateModel(NodeModel model){
			model.getProperty().setAppearanceGroup((AppearanceGroup)comboBox.getSelectedItem());
		}
	}
	
	private class CommentPanel extends AbstractPropertyPanel{
		
		private JTextArea textArea;
		
		public CommentPanel() {
			super();
			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			setBorder(new TitledBorder(Messages.getString("NodePropertyPanel.comment"))); //$NON-NLS-1$
			
			textArea = new JTextArea();
			add(new JScrollPane(textArea));
		}
		
		public void setModel(NodeModel model) {
			if(model.getProperty().getComment() != null)
				textArea.setText(model.getProperty().getComment());
		}
		
		public void updateModel(NodeModel model){
			String comment = textArea.getText();
			if(comment != null && comment.length() < 1)
				comment = null;
			model.getProperty().setComment(comment);
		}
	}
}
