package com.ef_prime.rflow.node.base;

import java.awt.Dimension;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;

import com.ef_prime.rflow.Prefs;
import com.ef_prime.rflow.exception.ModelCreationFailedException;
import com.ef_prime.rflow.node.DefaultNodeModelController;
import com.ef_prime.rflow.node.NodeModel;
import com.ef_prime.rflow.node.NodeOptionPanel;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.tool.CodingTextPane;

public class FreeNodeModelController extends DefaultNodeModelController{
	
	public String getModelName() {
		return Messages.getString("FreeNodeModelController.model_name"); //$NON-NLS-1$
	}
	
	public Class<? extends NodeModel> getModelClass() {
		return FreeNodeModel.class;
	}
	
	public Properties getOptionProperties(NodeModel model) {
		return null;
	}
	
	public NodeModel createModel(Properties properties, String title, String command) throws ModelCreationFailedException {
		FreeNodeModel model = new FreeNodeModel();
		model.setTitle(title);
		model.command = command;
		return model;
	}	

	@Override
	protected NodeOptionPanel createNodeOptionPanel(NodeModel model) {
		return new FreeNodeOptionPanel(model);
	}

	
	private static class FreeNodeOptionPanel extends NodeOptionPanel{

		FreeNodeModel model;
		List<UpdateModel> modelUpdaters = new ArrayList<UpdateModel>();
		
		FreeNodeOptionPanel(NodeModel model){
			super();
			this.model = (FreeNodeModel)model;
			
			TitlePanel titlePanel = new TitlePanel();
			CodePanel codePanel = new CodePanel();
			
			modelUpdaters.add(titlePanel);
			modelUpdaters.add(codePanel);

			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			int space = 20;
			setBorder(new EmptyBorder(space,space,space,space));
			add(titlePanel);
			add(Box.createVerticalStrut(DEFAULT_SPACE));
			add(codePanel);
		}
		
		@Override
		protected NodeModel getModel() {
			for(UpdateModel updatemodel : modelUpdaters){
				updatemodel.updateModel();
			}
			return model;
		}

		private class TitlePanel extends JPanel implements UpdateModel{

			JTextField textField = new JTextField();
			
			TitlePanel(){
				setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
				
				//タイトル
				JLabel titleLabel = new JLabel(Messages.getString("FreeNodeModelController.title")); //$NON-NLS-1$
				titleLabel.setMaximumSize(DEFAULT_LABEL_SIZE);
				titleLabel.setPreferredSize(DEFAULT_LABEL_SIZE);
				
				//
				textField.setText(model.getTitle());
				textField.setMaximumSize(new Dimension(1000, DEFAULT_LABEL_SIZE.height));
				
				add(titleLabel);
				add(textField);
			}
			
			public void updateModel() {
				model.setTitle(textField.getText());
			}
			
		}
		
		private class CodePanel extends JPanel implements UpdateModel{
			
			CodingTextPane textPane;
			//JEditorPane textPane = new JEditorPane();
			
			CodePanel(){
				setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
				
				
				//label
				JLabel codeLabel = new JLabel(Messages.getString("FreeNodeModelController.code")); //$NON-NLS-1$
				codeLabel.setMaximumSize(DEFAULT_LABEL_SIZE);
				codeLabel.setPreferredSize(DEFAULT_LABEL_SIZE);
				
				Box labelBox = Box.createHorizontalBox();
				labelBox.add(codeLabel);
				labelBox.add(Box.createHorizontalGlue());
				
				//		
				textPane = new CodingTextPane(model.getCommand());
				textPane.setFont(Prefs.CODE_FONT);
				textPane.setPreferredSize(new Dimension(500, 300));
				JScrollPane sp = new JScrollPane(textPane);
				
				//button
				/*JButton check = new JButton("チェック");
				check.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent e) {
						String command = textPane.getRawText();
						String function = "function(){" + command + "}";
						RCommand.checkUsage(function);

						String[] variables = RCommand.getLocalVariables(function);
						if(variables == null)return;
						for(String variable : variables){
							System.out.println(variable);
						}
					}
				});
				Box checkBox = Box.createHorizontalBox();
				checkBox.add(Box.createHorizontalGlue());
				checkBox.add(check);*/

				add(labelBox);
				add(sp);
				//add(checkBox);
			}
			
			public void updateModel() {
				model.command = textPane.getRawText();
			}
		}
		
	}
}
